{% extends "terminusgps_payments/layout.html" %}
{% partialdef main %}
<form method="post" action="{% url 'terminusgps_payments:delete subscriptions' subscription.pk %}">
    {% csrf_token %}
    <p>Are you sure you want to cancel '{{ subscription.name }}'?</p>
    {{ form }}
    <button type="submit">Confirm</button>
</form>
{% endpartialdef main %}
{% block content %}
{% partial main %}
{% endblock content %}
