{% extends "terminusgps_payments/layout.html" %}
{% block title %}Create Address Profile{% endblock title %}
{% partialdef main %}
<form action="{% url 'terminusgps_payments:create address profiles' %}" method="post">
    {% csrf_token %}
    {{ form }}
    <button type="submit">Submit</button>
</form>
{% endpartialdef main %}
{% block content %}
{% partial main %}
{% endblock content %}
