{% extends "terminusgps_payments/layout.html" %}
{% block title %}Address Profile Details{% endblock title %}
{% partialdef main %}
{{ customeraddressprofile }}
{% endpartialdef main %}
{% block content %}
{% partial main %}
{% endblock content %}
