{% extends "terminusgps_payments/layout.html" %}
{% block title %}List Address Profiles{% endblock title %}
{% partialdef main %}
<ul class="list-disc list-inside">
{% for profile in object_list %}
    <a
        class="hover:underline"
        href="{{ profile.get_absolute_url }}"
    >
        <li>{{ profile }}</li>
    </a>
{% empty %}
    <p>You don't have any address profiles.</p>
{% endfor %}
</ul>
{% endpartialdef main %}
{% block content %}
{% partial main %}
{% endblock content %}
