{% extends "terminusgps_payments/layout.html" %}
{% block title %}Payment Profile Details{% endblock title %}
{% partialdef main %}
{{ customerpaymentprofile }}
{% endpartialdef main %}
{% block content %}
{% partial main %}
{% endblock content %}
