{% extends "terminusgps_payments/layout.html" %}
{% block title %}List Payment Profiles{% endblock title %}
{% partialdef main %}
{% for profile in object_list %}
<a href="{{ profile.get_absolute_url }}">{{ profile.pk }}</a>
{% empty %}
<p>You don't have any payment profiles.</p>
{% endfor %}
{% endpartialdef main %}
{% block content %}
{% partial main %}
{% endblock content %}
